% '#######################################
% 'Slug_flow_dpdx_global
% '#######################################
function [DPDX, holdup] = dpdx_slug_unit(DLD, HfD, A, d, RHOL, RHOg, beta, AlfaS, AlfaU, LS, Lu, US, VISL, VISG, UF)
% 'Module for calculating pressure gradient for slug flow (intermittent flow regime)
% '
% 'Checking for slug flow : If no other flow regimes has been identified, slug flow is assumed

% '
% '   Calculate pressure drop using global momentum balance on slug unit:
% '   Parameters required for calculating TAUs:
% '
% '       RHOms = mixture density in slug
% '       VISms = mixture viscosity in slug
            RHOms = AlfaS * RHOg + (1 - AlfaS) * RHOL;
            VISms = AlfaS * VISG + (1 - AlfaS) * VISL;
% '
% '       Res = mixture Reynolds number in slug
            Res = US * RHOms * d / VISms;

% '       Blasius friction factor coefficients and exponent:
            if Res < 1500 
                CS = 16;
                ns = 1; 
            else
                CS = 0.046;
                ns = 0.2;
            end

% '       fs = friction factor, slug
            Fs = CS * Res ^ (-ns);

% '       TAUs = shear stress in slug zone
            TAUs = Fs * 0.5 * RHOms * US ^ 2;

% '   Parameters required for calculating TAUF and TAUG:
% 
% '       Checking if film zone is to modelled as annular or stratified
        if (beta > 0.5) || (beta < (-1.5)) 
% '           Film zone modelled as annular flow:
            Rf = 1 - (1 - 2 * DLD) ^ 2;
            Delta = DLD * d;
            Dgf = d - 2 * Delta;
            Agf = 3.14 / 4 * Dgf ^ 2;
            ALf = A - Agf;
            SLF = 3.14 * d;
            Sgf = 0;
        else
% '           Film zone modelled as stratified flow:
            WF = 2 * HfD - 1;
            acosWF = acos(WF);
            ALf = 0.25 * d ^ 2 * (3.14 - acosWF + WF * sqrt(1 - WF ^ 2));
            Rf = ALf / A;
            Agf = A - ALf;
            SLF = d * (3.14 - acosWF);
            Sgf = 3.14 * d - SLF;
            SI = d * sqrt(1 - WF ^ 2);
            Dgf = 4 * Agf / (Sgf + SI);
        end

            DLF = 4 * ALf / SLF;
            Ug = (US - UF * Rf) / (1 - Rf);

% '       Reynolds no of liquid in liquid film:
            RelF = abs(UF) * DLF * RHOL / VISL;
% '       Reynolds no of gas in liquid film zone:
            Regf = abs(Ug) * Dgf * RHOg / VISG;

            if RelF < 1500 
                Cl = 16;
                nL = 1; 
            else
                Cl = 0.046;
                nL = 0.2;
            end
            if Regf < 1500 
                CG = 16;
                ng = 1;
            else
                CG = 0.046;
                ng = 0.2;
            end

% '       SHEAR STRESS BETWEEN LIQUID AND PIPE:
            TAULf = 0.5 * RHOL * abs(UF) * UF * Cl * RelF ^ (-nL);
% '       SHEAR STRESS BETWEEN GAS AND PIPE:
            if (Sgf == 0)
                TAUgf = 0;
            else
                TAUgf = 0.5 * RHOg * abs(Ug) * Ug * CG * Regf ^ (-ng);
            end 

% '       Mixture density in slug unit:
            RHOmu = (1 - AlfaU) * RHOL + AlfaU * RHOg;

      DPDX = RHOmu * 9.81 * sin(beta) + TAUs * (3.14 * d / A) * LS / Lu + (TAULf * SLF + TAUgf * Sgf) / A * (Lu - LS) / Lu;
      holdup = 1-AlfaU;
end