% Standalone Network Solver for Short-Term prodcution optimization

% All Comments:
% 1. All properties may be included into structures (prop, rates, wells etc.) for a cleaner look.
% 4. Current example data are mostly from Exercise 5, TPG4230.


clear;
clear global;
clc;
%% Specify global variables and add folders to path

restoredefaultpath;
addpath(genpath('../Standalone_Network_Solver_BB'))
format short g;
clc;

global dens_o_sc dens_w_sc gamma_o gamma_g_0 gamma_gl_0 visc_o_sc visc_g_sc_0 pSep pBP_0 h_tubing l_tubing ...
    h_flowline l_flowline h_riser l_riser rough M_g p_sc T_sc g R wc GOR_0 nWells API_o API_w M_air qgl_sc_max...
    inch bara day qo_sc_min l_PLEM_WH h_PLEM_WH nManifold wells_in_manifold ID_t ID_f ID_m ...
    Tbnoil Tbngas h_oil h_gas k_steel k_sand thickness_t thickness_m thickness_f rlm_t rlm_m rlm_f ro_t ro_m ro_f ...
    h_sea k_ins ro_ins_m ro_ins_f T_sea step_length

%% Conversion factors

inch = 0.0254; % [m]
bara = 10^5;   % [pa]
day = 24*3600; % [s]


%% Import production profiles and pRes over time.
% Self generated or from simulator (Eclipse/MRST/Remso) using some model.



alpha = 3; %Parallel
alpha = alpha';
for i = 1:length(alpha)
    qo_sc(i, :) = [1200]*alpha(i)/day;
    qo_target(i, 1) =  sum(qo_sc(i, :))*day;
    pR(i, 1) = 190;   % All pressures in [Bar]
    TR_C = 100;
    TR(i, 1) = TR_C+273.15;
end

wc = [0.3];


% You can also choose to define GOR_0, and from there receive the pBP_0
GOR_0 = 500;   % Sm3/sm3, The "0"-values, eg. GOR_0 is value at reservoir condition.

qw_sc = zeros(size(qo_sc));
qg_sc = zeros(size(qo_sc));
qgl_sc = zeros(size(qo_sc));
dp_choke = zeros(size(qo_sc));

for k =1:length(pR)
qw_sc(k, :) = qo_sc(k, :).*wc; 
qg_sc(k, :) = qo_sc(k, :).*GOR_0;
end

qgl_sc_max = 50000/day;
qo_sc_min = 0.1/day;

%% Specify reservoir properties and constants

pSep = 7;
p_sc = 1.01325;
T_sc = 15.56+273.15;

g = 9.81; % [m/s^2]
R = 8314;  % [J/Kmol]



% As long as rates, pRes and time are given, the rest of the program works.
%% Specify fluid properties (Gas without GL)

% At SC
dens_o_sc = 780; % [kg/Sm3]
dens_w_sc = XSteam('rho_pT', p_sc, T_sc-273.15);
M_g = 19;  % [kg/kg-mole]
M_gl = 25;
M_air = 28.97;
gamma_g_0 = M_g/M_air;
gamma_gl_0 = M_gl/M_air;
gamma_o = dens_o_sc/dens_w_sc;
gamma_w = dens_w_sc/dens_w_sc;
API_o = 141.5/gamma_o-131.5;
API_w = 141.5/gamma_w-131.5;

pBP_0 = bubble_point(GOR_0, TR(k), gamma_g_0);

% At SC, "Dead o/w/g". Should depend upon sep. Conditions?
visc_o_sc = dead_visc_oil(T_sc, API_o);   % [cp]
%visc_w_sc = dead_visc_oil(T_sc, API_w);   % [cp]

visc_g_sc_0 = dead_visc_gas(gamma_g_0);

% Boiling points of the gas and liquid mixture:
Tbnoil = 851;   % Rankine
Tbngas = 196.47; % Rankine

% Convection heat transfer coefficients for oil and gas [W/m2 K]:
h_oil = 50;
h_gas = 11;

% Thermal conductivity of Steel and Sandstone
k_steel = 43; % (Stainless = 17. With 1% carbon, it's 43)
k_sand = 1.7;

%% Specify network properties
% May be tweeked for multiple wells, of course.

nWells = length(wc);
wellName =zeros(nWells, 1);
for i=1:nWells
wellName(i, 1) = i;
end

h_tubing = [2000]; % TVD [m]
l_tubing = [2000]; % Measured Length [m]
wells_in_manifold = [1]; % Number of wells in [first, second, ..., n.th] manifold.
l_PLEM_WH = [800]; % Distance from PLEM to manifold 1 and 2. 2 Wells for each manifold.
nManifold = length(l_PLEM_WH);
h_PLEM_WH = [30];
h_flowline = 400;
l_flowline = 3000;
h_riser = 100;
l_riser = 100;


% for i = 1:nManifold
%     temp_10=0;
%     for y = 1:i
%         temp_9 = temp_10;
%         temp_10 = temp_10+wells_in_manifold(y);    
%     end
%     TVD(temp_9+1:temp_10) = h_tubing(temp_9+1:temp_10) + h_PLEM_WH(i) + h_flowline + h_riser;
% 
% end


ID_t = 5*inch; % Tubing ID [inch], for all pipes
ID_m = (wells_in_manifold.^0.5)*ID_t;
ID_f = ID_t*sqrt(nWells);

% Pipes thickness:
thickness_t = 0.5*inch; % [m]
thickness_m = 1*inch; % [m]
thickness_f = 1.15*inch; % [m]

% Outer radius [m]:
ro_t = ID_t/2 + thickness_t; 
ro_m = ID_m/2 + thickness_m; 
ro_f = ID_f/2 + thickness_f; 

rlm_t = thickness_t/( log(ro_t) - log(ID_t/2));
rlm_m = thickness_m./( log(ro_m) - log(ID_m/2));
rlm_f = thickness_f/( log(ro_f) - log(ID_f/2));

% HTC in the sea:
% Outer radius of insulation [m]:
ro_ins_m = ro_m + 0.033;
ro_ins_f = ro_f + 0.033;
% Conductivity of insulation [W/m K]:
k_ins = 0.22;
% Sea convection coefficient [W/m2 K]:
h_sea = 200;
T_sea = 6+273.15;

rough = 2.54*10^-5; % Roughness For all pipes, not needed for Beggs&Brill model [m]
%% Flow in pipes, DP calculations
time=[];
% Choose solver: 
choice = menu('Choose an optimizer','Sequential Approach','Parallel Approach');


% For running simulations with multiple, specified pR and rates.


for k=1:length (pR)
    disp(alpha(k));
tic;
if sum(qo_sc(k, :)  )+sum(qw_sc(k, :)   )==0  % Gas production only, use compressors
        res = gasProd(qg_sc(k, :), pR(k));
else  % Multiphase flow, use GL or ESP
    if choice == 1
        [pTubing, pPLEM_WH, pRiser_Flow, p, T, qo_sc(k, :), qgl_sc(k, :), dp_choke(k, :)] = GL_sequential_NR(qo_sc(k, :), pR(k), TR(k));
    elseif choice == 2
        [pTubing, pPLEM_WH, pRiser_Flow, p, qo_sc(k, :), qg_sc(k, :), qw_sc(k, :), qgl_sc(k, :), dp_choke(k, :)] = GL_parallel_NR(qo_sc(k, :), qw_sc(k, :), qg_sc(k, :), pR(k));
    end
end
time(k, 1) = toc;
qo_sum(k, 1) = sum(qo_sc(k, :));
GL_sum(k, 1) = sum(qgl_sc(k, :));

% fprintf('Simulation runtime [s]: %.2f. \nField Oil rate [Sm3/D]: %.2f \nField Gas Lift rate [Sm3/D]: %.2f \n\n', time(k, 1), qo_sum(k, 1), GL_sum(k, 1));
% %% Plot of results:
% 
% % Flowline and riser:
% pRiser_Flow_plot = [];
%     for j = length(pRiser_Flow(:, 1)):-1:1
%         if pRiser_Flow(j, 1)>0
%             pRiser_Flow_plot = [pRiser_Flow_plot, pRiser_Flow(j, 1)];   % Pressure values from Sep. to Bottom.
%         end
%     end
% lRiser_Flow_plot = (0:(length(pRiser_Flow_plot)-1))*step_length;
% 
% 
% figure(k)
% hold on;
% cmap = hsv(nWells);
% 
% for i = 1:nWells
%     
%     for z = 1:nManifold
%         temp_10=0;
%         for y = 1:z
%             temp_9 = temp_10;
%             temp_10 = temp_10+wells_in_manifold(y);
%         end
%         if i>temp_9 && i<=temp_10
%             % PLEM-WH:
%             pPLEM_WH_plot = [];
%             for j = length(pPLEM_WH(:, z)):-1:1
%                 if pPLEM_WH(j, z)>0
%                     pPLEM_WH_plot = [pPLEM_WH_plot, pPLEM_WH(j, z)];   % Pressure values from Sep. to Bottom.
%                 end
%             end
%             lPLEM_WH_plot = lRiser_Flow_plot(end):step_length:(lRiser_Flow_plot(end)+(length(pPLEM_WH_plot)-1)*step_length);
%             break;
%         end
%         
%     end
%     
%     % Tubing:
%     pTubing_plot = [];
%     for j = length(pTubing(:, i)):-1:1
%         if pTubing(j, i)>0
%             pTubing_plot = [pTubing_plot, pTubing(j, i)];   % Pressure values from Sep. to Bottom.
%         end
%     end
%     lTubing_plot = lPLEM_WH_plot(end):step_length:(lPLEM_WH_plot(end)+(length(pTubing_plot)-1)*step_length);
%     % Choke:
%     pChoke_plot = [pPLEM_WH_plot(end) pTubing_plot(1)];
%     lChoke_plot = [lPLEM_WH_plot(end) lPLEM_WH_plot(end)];
%     % Total plot:
%     lPlot = [lRiser_Flow_plot, lPLEM_WH_plot, lChoke_plot, lTubing_plot];
%     pPlot = [pRiser_Flow_plot, pPLEM_WH_plot, pChoke_plot, pTubing_plot];
%     
%     plot( lPlot, pPlot, 'Color', cmap(i, :) );
%     
%     %str = num2str(i);
%     legendInfo{i} = ['Well = ' num2str(i)];
% end
% legend(legendInfo, 'Location', 'NorthWest');
% title(['Pressure along pipe at time ', num2str(k)]);
% xlabel('Travel distance from Sep. [m]');
% ylabel('Pressure [bar]');
% hold off;

end


